from typing import Optional

from django.utils.translation import gettext as _
from wbcore.contrib.color.enums import WBColor
from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    Display,
    create_simple_display,
    create_simple_section,
)
from wbcore.metadata.configs.display.instance_display.utils import repeat_field
from wbcore.metadata.configs.display.view_config import DisplayViewConfig

from wbmailing import models


class MailingListDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="title", label=_("Title")),
                dp.Field(key="nb_subscribers", label=_("Subscribers")),
                dp.Field(key="is_public", label=_("Public")),
            ]
        )

    def get_instance_display(self) -> Display:
        return create_simple_display(
            [["title", "nb_subscribers", "is_public"], [repeat_field(3, "email_contacts_section")]],
            [
                create_simple_section(
                    "email_contacts_section",
                    _("Email Contacts"),
                    [["email_contacts"]],
                    "email_contacts",
                    collapsed=True,
                )
            ],
        )


class MailingListSubscriberChangeRequestDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="type", label=_("Type")),
                dp.Field(key="email_contact", label=_("Email Contact")),
                dp.Field(key="entry_repr", label=_("Entry")),
                dp.Field(key="requester", label=_("Requester")),
                dp.Field(key="mailing_list", label=_("Mailing List")),
                dp.Field(key="created", label=_("Creation Date")),
                dp.Field(key="expiration_date", label=_("Expiration Date")),
                dp.Field(key="reason", label=_("Reason")),
            ],
            legends=[
                dp.Legend(
                    key="status",
                    items=[
                        dp.LegendItem(
                            icon=WBColor.YELLOW_LIGHT.value,
                            label=models.MailingListSubscriberChangeRequest.Status.PENDING.label,
                            value=models.MailingListSubscriberChangeRequest.Status.PENDING,
                        ),
                        dp.LegendItem(
                            icon=WBColor.GREEN_LIGHT.value,
                            label=models.MailingListSubscriberChangeRequest.Status.APPROVED.label,
                            value=models.MailingListSubscriberChangeRequest.Status.APPROVED,
                        ),
                        dp.LegendItem(
                            icon=WBColor.RED_LIGHT.value,
                            label=models.MailingListSubscriberChangeRequest.Status.DENIED.label,
                            value=models.MailingListSubscriberChangeRequest.Status.DENIED,
                        ),
                    ],
                )
            ],
            formatting=[
                dp.Formatting(
                    column="status",
                    formatting_rules=[
                        dp.FormattingRule(
                            style={"backgroundColor": WBColor.YELLOW_LIGHT.value},
                            condition=(
                                "==",
                                models.MailingListSubscriberChangeRequest.Status.PENDING,
                            ),
                        ),
                        dp.FormattingRule(
                            style={"backgroundColor": WBColor.GREEN_LIGHT.value},
                            condition=(
                                "==",
                                models.MailingListSubscriberChangeRequest.Status.APPROVED,
                            ),
                        ),
                        dp.FormattingRule(
                            style={"backgroundColor": WBColor.RED_LIGHT.value},
                            condition=(
                                "==",
                                models.MailingListSubscriberChangeRequest.Status.DENIED,
                            ),
                        ),
                    ],
                )
            ],
        )

    def get_instance_display(self) -> Display:
        return create_simple_display(
            [
                [repeat_field(3, "status")],
                ["email_contact", "mailing_list", "expiration_date"],
                [
                    "requester",
                    "approver",
                    ".",
                ],
                [
                    "created",
                    "updated",
                    ".",
                ],
                [repeat_field(3, "reason")],
            ]
        )


class MailingListSubscriberRequestMailingListDisplayConfig(MailingListSubscriberChangeRequestDisplayConfig):
    def get_instance_display(self) -> Display:
        return create_simple_display(
            [
                [repeat_field(2, "status")],
                ["expiration_date", "email_contact"],
            ]
        )


class EmailContactMailingListDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="status", label=_("Status")),
                dp.Field(key="email_contact", label=_("Email Contact")),
                dp.Field(key="expiration_date", label=_("Expiration Date")),
                dp.Field(key="is_pending_request_change", label=_("Pending Change")),
            ]
        )


class MailingListEntryDisplayConfig(MailingListDisplayConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="status", label=_("Status")),
                dp.Field(key="mailing_list", label=_("Mailing List")),
                dp.Field(key="is_public", label=_("Public")),
                dp.Field(key="expiration_date", label=_("Expiration Date")),
                dp.Field(key="is_pending_request_change", label=_("Pending Change")),
            ]
        )


class MailingListEntryCompanyDisplayConfig(MailingListEntryDisplayConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="title", label=_("Mailing List")),
                dp.Field(key="is_public", label=_("Public")),
            ]
        )


class MailingListSubscriberRequestEntryDisplayConfig(MailingListSubscriberChangeRequestDisplayConfig):
    def get_instance_display(self) -> Display:
        return create_simple_display(
            [[repeat_field(3, "status")], ["email_contact", "mailing_list", "expiration_date"]]
        )



from django.utils.translation import gettext as _
from wbcore.metadata.configs.titles import TitleViewConfig

from wbmailing import models


class MassMailTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Mass Mail: {{subject}}")

    def get_list_title(self):
        return _("Mass Mails")

    def get_create_title(self):
        return _("New Mass Mail")


class MailTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Mail: {{subject}}")

    def get_list_title(self):
        return _("Mails")

    def get_create_title(self):
        return _("Write E-Mail")


class MailStatusMassMailTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Mail: {{subject}}")

    def get_list_title(self):
        massmail = models.MassMail.objects.get(id=self.view.kwargs["mass_mail_id"])
        return _("Mails' Status for {}").format(massmail.subject)


class MailTemplateTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Mail Template: {{title}}")

    def get_list_title(self):
        return _("Mail Templates")


class MailEventMailTitleConfig(TitleViewConfig):
    def get_list_title(self):
        mail = models.Mail.objects.get(id=self.view.kwargs["mail_id"])
        emails = ", ".join([email.address for email in mail.to_email.all()])
        return _("Event for Mail {} to {} ({})").format(mail.subject, emails, mail.created)


class MailEventMassMailTitleConfig(TitleViewConfig):
    def get_list_title(self):
        massmail = models.MassMail.objects.get(id=self.view.kwargs["mass_mail_id"])
        return _("Event for Mass Mail {}").format(str(massmail))



from datetime import timedelta

from django.utils import timezone
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.registries import global_preferences_registry
from rest_framework import serializers
from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.contrib.directory.models import EmailContact
from wbcore.contrib.directory.serializers import (
    EmailContactRepresentationSerializer,
    EntryRepresentationSerializer,
    PersonRepresentationSerializer,
)

from wbmailing import models

from .mailing_lists import MailingListRepresentationSerializer


class MailRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbmailing:mail-detail")

    class Meta:
        model = models.Mail
        fields = ("id", "subject", "_detail")


class MassMailRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbmailing:massmail-detail")

    class Meta:
        model = models.MassMail
        fields = ("id", "subject", "_detail")


class MailTemplateRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbmailing:mailtemplate-detail")

    class Meta:
        model = models.MailTemplate
        fields = ("id", "title", "_detail")


class MassMailListSerializer(wb_serializers.ModelSerializer):
    _creator = PersonRepresentationSerializer(source="creator")
    _mailing_lists = MailingListRepresentationSerializer(label=_("Mailing List"), source="mailing_lists", many=True)
    _excluded_mailing_lists = MailingListRepresentationSerializer(
        label=_("Excluded Mailing List"), source="excluded_mailing_lists", many=True
    )
    _template = MailTemplateRepresentationSerializer(source="template")
    subject = wb_serializers.CharField(required=True, label=_("Subject"))

    class Meta:
        model = models.MassMail
        fields = (
            "id",
            "status",
            "mailing_lists",
            "_mailing_lists",
            "excluded_mailing_lists",
            "_excluded_mailing_lists",
            "from_email",
            "template",
            "_template",
            "subject",
            "created",
            "creator",
            "_creator",
            "send_at",
        )


class MassMailModelSerializer(MassMailListSerializer):
    from_email = wb_serializers.CharField(
        default=lambda: global_preferences_registry.manager()["wbmailing__default_source_mail"], label=_("From")
    )

    @wb_serializers.register_resource()
    def send_test_mail(self, instance, request, user):
        return {
            "send_test_mail": reverse(
                "wbmailing:massmail-sendtestmail",
                args=[instance.id],
                request=request,
            )
        }

    @wb_serializers.register_resource()
    def mails(self, instance, request, user):
        return {
            "mails": reverse(
                "wbmailing:massmail-mailstatus-list",
                args=[instance.id],
                request=request,
            ),
            "events": reverse(
                "wbmailing:massmail-mailevent-list",
                args=[instance.id],
                request=request,
            ),
        }

    @wb_serializers.register_resource()
    def analytics(self, instance, request, user):
        return {
            "mailstatus_barchart": reverse(
                "wbmailing:massmail-mailstatusbarchart-list", args=[instance.id], request=request
            ),
            "mailclick_barchart": reverse(
                "wbmailing:massmail-mailclickbarchart-list", args=[instance.id], request=request
            ),
            "clients_barchart": reverse(
                "wbmailing:massmail-clientsbarchart-list", args=[instance.id], request=request
            ),
            "country_barchart": reverse(
                "wbmailing:massmail-countrybarchart-list", args=[instance.id], request=request
            ),
            "region_barchart": reverse("wbmailing:massmail-regionbarchart-list", args=[instance.id], request=request),
        }

    def create(self, validated_data, *args, **kwargs):
        if request := self.context.get("request"):
            validated_data["creator"] = request.user.profile
        return super().create(validated_data, *args, **kwargs)

    class Meta:
        model = models.MassMail
        fields = (
            "id",
            "status",
            "mailing_lists",
            "_mailing_lists",
            "excluded_mailing_lists",
            "_excluded_mailing_lists",
            "from_email",
            "template",
            "_template",
            "subject",
            "created",
            "body",
            "creator",
            "_creator",
            "send_at",
            "attachment_url",
            "_additional_resources",
        )

    def validate(self, data):
        send_at = data.get("send_at", None)
        if send_at:
            if send_at < timezone.now() - timedelta(minutes=1):
                raise serializers.ValidationError(
                    {"non_field_errors": gettext("Send time shouldn't be earlier than now.")}
                )
        return super().validate(data)


class MailModelSerializer(wb_serializers.ModelSerializer):
    from_email = wb_serializers.CharField(
        default=wb_serializers.CurrentUserDefault(user_attr="email"), label=_("From")
    )
    created = wb_serializers.DateTimeField(default=lambda: timezone.now(), label=_("Created"))
    _mass_mail = MassMailRepresentationSerializer(many=False, source="mass_mail")
    _template = MailTemplateRepresentationSerializer(many=False, source="template")
    _to_email = EmailContactRepresentationSerializer(many=True, source="to_email")
    _cc_email = EmailContactRepresentationSerializer(many=True, source="cc_email")
    _bcc_email = EmailContactRepresentationSerializer(many=True, source="bcc_email")

    status = wb_serializers.ChoiceField(
        choices=models.MailEvent.EventType.choices, default=models.MailEvent.EventType.UNKNOWN, read_only=True
    )

    @wb_serializers.register_resource()
    def mail_event(self, instance, request, user):
        return {
            "mailevent": reverse(
                "wbmailing:mail-mailevent-list",
                args=[instance.id],
                request=request,
            ),
            "resend_mail": reverse(
                "wbmailing:mail-resend",
                args=[instance.id],
                request=request,
            ),
        }

    class Meta:
        model = models.Mail
        fields = (
            "id",
            "created",
            "last_send",
            "template",
            "_template",
            "message_ids",
            "mass_mail",
            "_mass_mail",
            "from_email",
            "to_email",
            "_to_email",
            "cc_email",
            "_cc_email",
            "bcc_email",
            "_bcc_email",
            "subject",
            "body",
            "status",
            "_additional_resources",
        )


class MailStatusMassMailModelSerializer(wb_serializers.ModelSerializer):
    mail_id = wb_serializers.CharField(read_only=True)
    _entry = EntryRepresentationSerializer(source="entry")
    status = wb_serializers.ChoiceField(
        choices=(*models.MailEvent.EventType.choices, ("NOT_SENT", gettext("Not Sent"))),
        default="NOT_SENT",
        read_only=True,
    )

    class Meta:
        model = EmailContact
        fields = ("id", "entry", "_entry", "address", "status", "mail_id")


class MailTemplateModelSerializer(wb_serializers.ModelSerializer):
    class Meta:
        model = models.MailTemplate
        fields = ("id", "title", "template")


class MailEventModelSerializer(wb_serializers.ModelSerializer):
    _mail = MailRepresentationSerializer(source="mail")

    class Meta:
        model = models.MailEvent
        fields = (
            "id",
            "mail",
            "_mail",
            "timestamp",
            "event_type",
            "reject_reason",
            "description",
            "recipient",
            "click_url",
            "ip",
            "user_agent",
        )
